@@ -6,7 +6,7 @@ import os
 import pickle
 import subprocess
 import sys
-from pylammpsmpi.commands import command_list, thermo_list
+from pylammpsmpi.commands import command_list, thermo_list, func_list, prop_list
 
 
 
@@ -21,35 +21,24 @@ __email__ = "janssen@mpie.de"
 __status__ = "production"
 __date__ = "Feb 28, 2020"
 
-class LammpsLibrary(object):
-    def __init__(self, cores=1, working_directory="."):
+
+class LammpsBase:
+    def __init__(self, cores=8, working_directory="."):
+        self.cores = cores
+        self.working_directory = working_directory
+
+    def start_process(self):
         executable = os.path.join(
             os.path.dirname(os.path.abspath(__file__)), "mpi", "lmpmpi.py"
         )
         self._process = subprocess.Popen(
-            ["mpiexec", "-n", str(cores), "python", executable],
+            ["mpiexec", "--oversubscribe", "-n", str(self.cores), "python", executable],
             stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
+            stderr=None,
             stdin=subprocess.PIPE,
-            cwd=working_directory,
+            cwd=self.working_directory,
         )
 
-    def __getattr__(self, name):
-        """
-        Try to run input as a lammps command
-        """
-        def command_wrapper(*args):
-            args = [name] + list(args)
-            cmd = " ".join([str(x) for x in args])
-            self.command(cmd)
-
-        if name in command_list:
-            return command_wrapper
-        elif name in thermo_list:
-            return self.get_thermo(name)
-        else:
-            raise AttributeError(name)
-
     def _send(self, command, data=None):
         """
         Send a command to the Lammps Library executable
@@ -118,6 +107,7 @@ class LammpsLibrary(object):
         if not os.path.exists(inputfile):
             raise FileNotFoundError("Input file does not exist")
         self._send(command="get_file", data=[inputfile])
+        _ = self._receive()
 
 
     #TODO
@@ -328,10 +318,54 @@ class LammpsLibrary(object):
             box tilts
         """
         self._send(command="reset_box", data=list(args))
+        _ = self._receive()
+
+    def generate_atoms(self, ids=None, type=None, x=None, v=None, image=None, shrinkexceed=False):
+        """
+        Create atoms on all procs
+
+        Parameters
+        ----------
+        ids : list of ints, optional
+            ids of N atoms that need to be created
+            if not specified, ids from 1 to N are assigned
+
+        type : list of atom types, optional
+            type of N atoms, if not specied, all atoms are assigned as type 1
+
+        x: list of positions
+            list of the type `[posx, posy, posz]` for N atoms
+
+        v: list of velocities
+            list of the type `[vx, vy, vz]` for N atoms
+
+        image: list of ints, optional
+            if not specified a list of 0s will be used.
+
+        shrinkexceed: bool, optional
+            default False
+
+        Returns
+        -------
+        None
+
+        """
+
+        if x is not None:
+            natoms = len(x)
+            if type is None:
+                type = [1]*natoms
+            if ids is None:
+                ids = list(range(1, natoms+1))
+            if image is None:
+                image = [0]*natoms
+
+            funct_args = [natoms, ids, type, x, v, image, shrinkexceed]
+            self._send(command="create_atoms", data=funct_args)
+            _ = self._receive()
+        else:
+            raise TypeError("Value of x cannot be None")
 
-    #TODO
-    def create_atoms(self, *args):
-        self._send(command="create_atoms", data=list(args))
 
     @property
     def has_exceptions(self):
@@ -454,11 +488,14 @@ class LammpsLibrary(object):
         if isinstance(cmd, list):
             for c in cmd:
                 self._send(command="command", data=c)
+                _ = self._receive()
         elif len(cmd.split('\n')) > 1:
             for c in cmd.split('\n'):
                 self._send(command="command", data=c)
+                _ = self._receive()
         else:
             self._send(command="command", data=cmd)
+            _ = self._receive()
 
 
 
@@ -521,8 +558,10 @@ class LammpsLibrary(object):
             args.append(len(ids))
             args.append(ids)
             self._send(command="scatter_atoms_subset", data=args)
+            _ = self._receive()
         else:
             self._send(command="scatter_atoms", data=list(args))
+            _ = self._receive()
 
     def get_thermo(self, *args):
         """
